/* Copyright (c) 1996-2013 Clickteam
 *
 * This source code is part of the Android exporter for Clickteam Multimedia Fusion 2.
 * 
 * Permission is hereby granted to any person obtaining a legal copy 
 * of Clickteam Multimedia Fusion 2 to use or modify this source code for 
 * debugging, optimizing, or customizing applications created with 
 * Clickteam Multimedia Fusion 2.  Any other use of this source code is prohibited.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package Extensions;

//----------------------------------------------------------------------------------
//
// CRunBitwiser: Bitwiser object
// Created by Danny Worth (2017)
// 
//----------------------------------------------------------------------------------

import Actions.CActExtension;
import Conditions.CCndExtension;
import Expressions.CValue;
import RunLoop.CCreateObjectInfo;
import Runtime.MMFRuntime;
import android.Manifest;
import Application.CRunApp;
import Objects.CObject;

public class CRunBitwiser extends CRunExtension
{
	static final int EXP_OR32 =						0;
    static final int EXP_XOR32 =					1;
    static final int EXP_AND32 =					2;
    static final int EXP_NOT32 =					3;
    static final int EXP_ROL32 =					4;
    static final int EXP_ROR32 =					5;
    static final int EXP_LSH32 =					6;
    static final int EXP_RSH32 =					7;
    static final int EXP_OR16 =						8;
    static final int EXP_XOR16 =					9;
    static final int EXP_AND16 =					10;
    static final int EXP_NOT16 =					11;
    static final int EXP_ROL16 =					12;
    static final int EXP_ROR16 =					13;
    static final int EXP_LSH16 =					14;
    static final int EXP_RSH16 =					15;
    static final int EXP_OR8 =						16;
    static final int EXP_XOR8 =						17;
    static final int EXP_AND8 =						18;
    static final int EXP_NOT8 =						19;
    static final int EXP_ROL8 =						20;
    static final int EXP_ROR8 =						21;
    static final int EXP_LSH8 =						22;
    static final int EXP_RSH8 =						23;
    static final int EXP_SETBITFLAG =				24;
    static final int EXP_CLEARBITFLAG =				25;
    static final int EXP_TOGGLEBITFLAG =			26;
    static final int EXP_EMBEDVALUE =				27;
    static final int EXP_RETRIEVEEMBEDDEDVALUE =	28;
    static final int EXP_SIGNED8 =					29;
    static final int EXP_SIGNED16 =					30;
	
	public CRunBitwiser() {
	}
	@Override 
	public int getNumberOfConditions()
	{
		return 1;
	}

	@Override
	public int handleRunObject()
	{
		return 0;
	}

	@Override
	public boolean condition(int num, CCndExtension cnd)
	{
		switch (num)
		{
		case 0:
			return IsFlagSet(cnd.getParamExpression(rh, 0), cnd.getParamExpression(rh, 1));
		}
		return false;
	}

	@Override
	public void action(int num, CActExtension act)
	{
	}

	@SuppressWarnings("deprecation")
	@Override
	public CValue expression(int num)
	{
		switch (num)
		{
            case EXP_OR32:
                return new CValue(OR32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_XOR32:
				return new CValue(XOR32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_AND32:
				return new CValue(AND32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_NOT32:
				return new CValue(NOT32(ho.getExpParam().getInt()));
			case EXP_ROL32:
				return new CValue(ROL32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_ROR32:
				return new CValue(ROR32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_LSH32:
				return new CValue(LSH32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_RSH32:
				return new CValue(RSH32(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_OR16:
				return new CValue(OR16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_XOR16:
				return new CValue(XOR16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_AND16:
				return new CValue(AND16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_NOT16:
				return new CValue(NOT16(ho.getExpParam().getInt()));
			case EXP_ROL16:
				return new CValue(ROL16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_ROR16:
				return new CValue(ROR16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_LSH16:
				return new CValue(LSH16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_RSH16:
				return new CValue(RSH16(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_OR8:
				return new CValue(OR8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_XOR8:
				return new CValue(XOR8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_AND8:
				return new CValue(AND8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_NOT8:
				return new CValue(NOT8(ho.getExpParam().getInt()));
			case EXP_ROL8:
				return new CValue(ROL8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_ROR8:
				return new CValue(ROR8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_LSH8:
				return new CValue(LSH8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_RSH8:
				return new CValue(RSH8(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_SETBITFLAG:
				return new CValue(SetBitFlag(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_CLEARBITFLAG:
				return new CValue(ClearBitFlag(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_TOGGLEBITFLAG:
				return new CValue(ToggleBitFlag(ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_EMBEDVALUE:
				return new CValue(EmbedValue(ho.getExpParam().getInt(), ho.getExpParam().getInt(), ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_RETRIEVEEMBEDDEDVALUE:
				return new CValue(RetrieveEmbeddedValue(ho.getExpParam().getInt(), ho.getExpParam().getInt(), ho.getExpParam().getInt()));
			case EXP_SIGNED8:
				return new CValue(Signed8(ho.getExpParam().getInt()));
			case EXP_SIGNED16:
				return new CValue(Signed16(ho.getExpParam().getInt()));
		}
		return new CValue(0);
	}
	
	private boolean IsFlagSet(int Flag, int Value)
	{
		int workingFlag = 1 << (Flag % 32);	
		return ((Value & workingFlag) != 0) ? true : false;
	}
	
	private static int GetBitMask(int bits)
	{
		int mask = 0;
		for (int i = 0; i < bits; i++)
		{
			mask <<= 1;
			mask |= 1;
		}
		return mask;
	}
	
	private static int OR32(int FirstValue, int SecondValue)
	{
		return FirstValue | SecondValue;
	}
	
	private static int OR16(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFFFF) | (SecondValue & 0xFFFF);
	}
	
	private static int OR8(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFF) | (SecondValue & 0xFF);
	}

	private static int XOR32(int FirstValue, int SecondValue)
	{
		return FirstValue ^ SecondValue;
	}
	
	private static int XOR16(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFFFF) ^ (SecondValue & 0xFFFF);
	}
	
	private static int XOR8(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFF) ^ (SecondValue & 0xFF);
	}
	
	private static int AND32(int FirstValue, int SecondValue)
	{
		return FirstValue & SecondValue;
	}
	
	private static int AND16(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFFFF) & (SecondValue & 0xFFFF);
	}
	
	private static int AND8(int FirstValue, int SecondValue)
	{
		return (FirstValue & 0xFF) & (SecondValue & 0xFF);
	}
	
	private static int NOT32(int Value)
	{
		return ~Value;
	}
	
	private static int NOT16(int Value)
	{
		return ~Value & 0xFFFF;
	}
	
	private static int NOT8(int Value)
	{
		return ~Value & 0xFF;
	}
	
	private static int ROL32(int Value, int Amount)
	{
		return Value << (Amount % 32) | Value >>> (32-(Amount % 32));
	}
	
	private static int ROL16(int Value, int Amount)
	{
		return (((Value & 0xFFFF) << (Amount % 16)) | ((Value & 0xFFFF) >>> (16-(Amount % 16)))) & 0xFFFF;
	}
	
	private static int ROL8(int Value, int Amount)
	{
		return (((Value & 0xFF) << (Amount % 8)) | ((Value & 0xFF) >>> (8-(Amount % 8)))) & 0xFF;
	}
	
	private static int ROR32(int Value, int Amount)
	{
		return Value >>> (Amount % 32) | Value << (32-(Amount % 32));
	}
	
	private static int ROR16(int Value, int Amount)
	{
		return (((Value & 0xFFFF) >>> (Amount % 16)) | ((Value & 0xFFFF) << (16-(Amount % 16)))) & 0xFFFF;
	}
	
	private static int ROR8(int Value, int Amount)
	{
		return (((Value & 0xFF) >>> (Amount % 8)) | ((Value & 0xFF) << (8-(Amount % 8)))) & 0xFF;
	}
	
	private static int LSH32(int Value, int Amount)
	{
		return Value << Amount;
	}
	
	private static int LSH16(int Value, int Amount)
	{
		return (Value << Amount) & 0xFFFF;
	}
	
	private static int LSH8(int Value, int Amount)
	{
		return (Value << Amount) & 0xFF;
	}
	
	private static int RSH32(int Value, int Amount)
	{
		return Value >>> Amount;
	}
	
	private static int RSH16(int Value, int Amount)
	{
		return (Value >>> Amount) & 0xFFFF;
	}
	
	private static int RSH8(int Value, int Amount)
	{
		return (Value >>> Amount) & 0xFF;
	}
	
	private static int SetBitFlag(int Value, int Bit)
	{
		int workingBit = 1 << (Bit % 32);
		return Value | workingBit;
	}
	
	private static int ClearBitFlag(int Value, int Bit)
	{
		int workingBit = 1 << (Bit % 32);
		return Value & (~workingBit);
	}

	private static int ToggleBitFlag(int Value, int Bit)
	{
		int workingBit = 1 << (Bit % 32);
		return Value ^ workingBit;
	}
	
	private static int EmbedValue(int Value, int EmbedValue, int BitOffset, int BitLength)
	{
		int BitMask = GetBitMask(BitLength);
		EmbedValue = (EmbedValue & BitMask) << BitOffset;
		BitMask <<= BitOffset;
		Value &= ~BitMask;
		Value |= EmbedValue;
		return Value;
	}

	private static int RetrieveEmbeddedValue(int Value, int BitOffset, int BitLength)
	{
		int BitMask = GetBitMask(BitLength);
		Value >>= BitOffset;
		Value &= BitMask;
		return Value;
	}

	private static byte Signed8(int Value)
	{
		return (byte)Value;
	}

	private static short Signed16(int Value)
	{
		return (short)Value;
	}
}